# Generated by Django 3.2.6 on 2021-08-14 21:30

import uuid

import django.contrib.auth.models
import django.contrib.auth.validators
import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="NAAN",
            fields=[
                (
                    "naan",
                    models.PositiveBigIntegerField(primary_key=True, serialize=False),
                ),
                ("name", models.CharField(max_length=200)),
                ("description", models.TextField()),
                ("url", models.URLField()),
            ],
        ),
        migrations.CreateModel(
            name="Shoulder",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("shoulder", models.CharField(max_length=50)),
                ("name", models.CharField(max_length=200)),
                ("description", models.TextField()),
                (
                    "naan",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING, to="ark.naan"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Key",
            fields=[
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("active", models.BooleanField()),
                (
                    "naan",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="ark.naan"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Ark",
            fields=[
                (
                    "ark",
                    models.CharField(
                        editable=False,
                        max_length=200,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("shoulder", models.CharField(editable=False, max_length=50)),
                ("assigned_name", models.CharField(editable=False, max_length=100)),
                ("url", models.URLField()),
                ("metadata", models.TextField()),
                ("commitment", models.TextField()),
                (
                    "naan",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to="ark.naan",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="User",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="last login"
                    ),
                ),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                (
                    "username",
                    models.CharField(
                        error_messages={
                            "unique": "A user with that username already exists."
                        },
                        help_text="Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.",
                        max_length=150,
                        unique=True,
                        validators=[
                            django.contrib.auth.validators.UnicodeUsernameValidator()
                        ],
                        verbose_name="username",
                    ),
                ),
                (
                    "first_name",
                    models.CharField(
                        blank=True, max_length=150, verbose_name="first name"
                    ),
                ),
                (
                    "last_name",
                    models.CharField(
                        blank=True, max_length=150, verbose_name="last name"
                    ),
                ),
                (
                    "email",
                    models.EmailField(
                        blank=True, max_length=254, verbose_name="email address"
                    ),
                ),
                (
                    "is_staff",
                    models.BooleanField(
                        default=False,
                        help_text="Designates whether the user can log into this admin site.",
                        verbose_name="staff status",
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="Designates whether this user should be treated as active. Unselect this instead of deleting accounts.",
                        verbose_name="active",
                    ),
                ),
                (
                    "date_joined",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="date joined"
                    ),
                ),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "naan",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="ark.naan",
                    ),
                ),
                (
                    "user_permissions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Specific permissions for this user.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Permission",
                        verbose_name="user permissions",
                    ),
                ),
            ],
            options={
                "verbose_name": "user",
                "verbose_name_plural": "users",
                "abstract": False,
            },
            managers=[
                ("objects", django.contrib.auth.models.UserManager()),
            ],
        ),
    ]



import uuid

from django.contrib.auth.models import AbstractUser
from django.core.exceptions import ValidationError
from django.db import models


class Naan(models.Model):
    naan = models.PositiveBigIntegerField(primary_key=True)
    name = models.CharField(max_length=200)
    description = models.TextField()
    url = models.URLField()

    def __str__(self):
        return f"{self.name} - {self.naan}"


class User(AbstractUser):
    naan = models.ForeignKey(Naan, on_delete=models.PROTECT, null=True)

    def __str__(self):
        return self.username


class Key(models.Model):
    key = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    naan = models.ForeignKey(Naan, on_delete=models.CASCADE)
    active = models.BooleanField()

    def __str__(self):
        return f"Key-{self.naan.naan}-{self.key.hex[:8]}..."


class Shoulder(models.Model):
    shoulder = models.CharField(max_length=50)
    naan = models.ForeignKey(Naan, on_delete=models.DO_NOTHING)
    name = models.CharField(max_length=200)
    description = models.TextField()

    def __str__(self):
        return f"{self.naan.naan}{self.shoulder}"


class Ark(models.Model):
    ark = models.CharField(primary_key=True, max_length=200, editable=False)
    naan = models.ForeignKey(Naan, on_delete=models.DO_NOTHING, editable=False)
    shoulder = models.CharField(max_length=50, editable=False)
    assigned_name = models.CharField(max_length=100, editable=False)
    url = models.URLField(default="", blank=True)
    metadata = models.TextField(default="", blank=True)
    commitment = models.TextField(default="", blank=True)

    def clean(self):
        expected_ark = f"{self.naan.naan}{self.shoulder}{self.assigned_name}"
        if self.ark != expected_ark:
            raise ValidationError(f"expected {expected_ark} got {self.ark}")

    def __str__(self):
        return f"ark:/{self.ark}"



import json
import logging

from django.core.exceptions import ValidationError
from django.db import IntegrityError
from django.http import (
    Http404,
    HttpResponse,
    HttpResponseBadRequest,
    HttpResponseForbidden,
    HttpResponseNotAllowed,
    HttpResponseRedirect,
    HttpResponseServerError,
    JsonResponse,
)
from django.views.decorators.csrf import csrf_exempt

from ark.forms import MintArkForm, UpdateArkForm
from ark.models import Ark, Naan
from ark.utils import generate_noid, noid_check_digit, parse_ark

logger = logging.getLogger(__name__)


@csrf_exempt
def mint_ark(request):
    if request.method != "POST":
        return HttpResponseNotAllowed(permitted_methods=["POST"])

    try:
        unsafe_mint_request = json.loads(request.body.decode("utf-8"))
    except (json.JSONDecodeError, TypeError) as e:
        return HttpResponseBadRequest(e)

    mint_request = MintArkForm(unsafe_mint_request)

    if not mint_request.is_valid():
        return JsonResponse(mint_request.errors, status=400)

    # TODO: get rid of UUID for key
    # TODO: hash the keys and only show on creation
    bearer_token = request.META.get("HTTP_AUTHORIZATION")
    if not bearer_token:
        return HttpResponseForbidden()

    key = bearer_token.split()[-1]

    try:
        authorized_naan = Naan.objects.get(key__key=key)
    except Naan.DoesNotExist:
        return HttpResponseForbidden()
    except ValidationError as e:  # probably an invalid key
        return HttpResponseBadRequest(e)

    naan = mint_request.cleaned_data["naan"]
    shoulder = mint_request.cleaned_data["shoulder"]
    url = mint_request.cleaned_data["url"]
    metadata = mint_request.cleaned_data["metadata"]
    commitment = mint_request.cleaned_data["commitment"]

    if authorized_naan.naan != naan:
        return HttpResponseForbidden()

    ark, collisions = None, 0
    for _ in range(10):
        noid = generate_noid(8)
        base_ark_string = f"{naan}{shoulder}{noid}"
        check_digit = noid_check_digit(base_ark_string)
        ark_string = f"{base_ark_string}{check_digit}"
        try:
            ark = Ark.objects.create(
                ark=ark_string,
                naan=authorized_naan,
                shoulder=shoulder,
                assigned_name=f"{noid}{check_digit}",
                url=url,
                metadata=metadata,
                commitment=commitment,
            )
            break
        except IntegrityError:
            collisions += 1
            continue

    if not ark:
        msg = f"Gave up creating ark after {collisions} collision(s)"
        logger.error(msg)
        return HttpResponseServerError(msg)
    if ark and collisions > 0:
        logger.warning("Ark created after %d collision(s)", collisions)

    return JsonResponse({"ark": str(ark)})


@csrf_exempt
def update_ark(request):
    if request.method != "PUT":
        return HttpResponseNotAllowed(permitted_methods=["PUT"])

    try:
        unsafe_update_request = json.loads(request.body.decode("utf-8"))
    except (json.JSONDecodeError, TypeError) as e:
        return HttpResponseBadRequest(e)

    # TODO: test input data with wrong structure
    update_request = UpdateArkForm(unsafe_update_request)

    if not update_request.is_valid():
        return JsonResponse(update_request.errors, status=400)

    # TODO: get rid of UUID for key
    # TODO: hash the keys and only show on creation
    bearer_token = request.META.get("HTTP_AUTHORIZATION")
    if not bearer_token:
        return HttpResponseForbidden()

    key = bearer_token.split()[-1]

    try:
        # TODO: is key valid enough to pass here?
        authorized_naan = Naan.objects.get(key__key=key)
    except Naan.DoesNotExist:
        return HttpResponseForbidden()
    except ValidationError as e:
        return HttpResponseBadRequest(e)

    ark = update_request.cleaned_data["ark"]
    url = update_request.cleaned_data["url"]
    metadata = update_request.cleaned_data["metadata"]
    commitment = update_request.cleaned_data["commitment"]

    _, naan, assigned_name = parse_ark(ark)

    if authorized_naan.naan != naan:
        return HttpResponseForbidden()

    try:
        ark = Ark.objects.get(ark=f"{naan}/{assigned_name}")
    except Ark.DoesNotExist:
        raise Http404

    ark.url = url
    ark.metadata = metadata
    ark.commitment = commitment
    ark.save()

    return HttpResponse()


def resolve_ark(request, ark: str):
    # TODO: maybe just parse the ark in the urls.py re_path
    try:
        _, naan, assigned_name = parse_ark(ark)
    except ValueError as e:
        return HttpResponseBadRequest(e)
    try:
        ark_obj = Ark.objects.get(ark=f"{naan}/{assigned_name}")
        if not ark_obj.url:
            # TODO: return a template page for an ARK in progress
            raise Http404
        return HttpResponseRedirect(ark_obj.url)
    except Ark.DoesNotExist:
        try:
            naan_obj = Naan.objects.get(naan=naan)
            return HttpResponseRedirect(
                f"{naan_obj.url}/ark:/{naan_obj.naan}/{assigned_name}"
            )
        except Naan.DoesNotExist:
            resolver = "https://n2t.net"
            # TODO: more robust resolver URL creation
            return HttpResponseRedirect(f"{resolver}/ark:/{naan}/{assigned_name}")



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("ark", "0002_allow_blank_ark_model_fields"),
    ]

    operations = [
        migrations.RunSQL(
            sql="ALTER TABLE ark_ark ALTER COLUMN commitment SET DEFAULT ''"
        ),
        migrations.RunSQL(
            sql="ALTER TABLE ark_ark ALTER COLUMN metadata SET DEFAULT ''"
        ),
        migrations.RunSQL(sql="ALTER TABLE ark_ark ALTER COLUMN url SET DEFAULT ''"),
    ]



from django import forms
from django.core.exceptions import ValidationError

from ark.utils import parse_ark


def validate_shoulder(shoulder: str):
    if not shoulder.startswith("/"):
        raise ValidationError("Shoulders must start with a forward slash")


def validate_ark(ark: str):
    try:
        parse_ark(ark)
    except ValueError as e:
        raise ValidationError(f"Invalid ARK: {e}")


class MintArkForm(forms.Form):
    naan = forms.IntegerField()
    shoulder = forms.CharField(validators=[validate_shoulder])
    url = forms.URLField(required=False)
    metadata = forms.CharField(required=False)
    commitment = forms.CharField(required=False)


class UpdateArkForm(forms.Form):
    ark = forms.CharField(validators=[validate_ark])
    url = forms.URLField(required=False)
    metadata = forms.CharField(required=False)
    commitment = forms.CharField(required=False)



from typing import Tuple

import secrets

BETANUMERIC = "0123456789bcdfghjkmnpqrstvwxz"


def noid_check_digit(noid: str) -> str:
    """Calculate the check digit for an ARK.

    See: https://metacpan.org/dist/Noid/view/noid#NOID-CHECK-DIGIT-ALGORITHM
    """
    total = 0
    for pos, char in enumerate(noid, start=1):
        score = BETANUMERIC.find(char)
        if score > 0:
            total += pos * score
    remainder = total % 29  # 29 == len(BETANUMERIC)
    return BETANUMERIC[remainder]  # IndexError may be long ARK


def generate_noid(length: int) -> str:
    return "".join(secrets.choice(BETANUMERIC) for _ in range(length))


def parse_ark(ark: str) -> Tuple[str, int, str]:
    parts = ark.split("ark:")
    if len(parts) != 2:
        raise ValueError("Not a valid ARK")
    nma, ark = parts
    ark = ark.lstrip("/")
    parts = ark.split("/")
    if len(parts) < 2:
        raise ValueError("Not a valid ARK")
    # TODO: assigned_name definition here conflicts with models.Ark.assigned_name
    # parse_ark here doesn't attempt to parse out the shoulder of the ARK.
    naan, assigned_name = parts[:2]
    try:
        naan_int = int(naan)
    except ValueError:
        raise ValueError("ARK NAAN must be an integer")

    return nma, naan_int, assigned_name



# Generated by Django 3.2.8 on 2022-02-02 21:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ark", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="ark",
            name="commitment",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="ark",
            name="metadata",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="ark",
            name="url",
            field=models.URLField(blank=True, default=""),
        ),
    ]



"""Django Admin command to mint ARKs in bulk.

This command was created to populate a test database for load testing. It only mints
ARKs, it doesn't bind URLs to them.
"""

from django.core.management.base import BaseCommand

from ark.models import Ark, Naan
from ark.utils import generate_noid


class Command(BaseCommand):
    """Mint ark_count ARKs for the given naan and shoulder."""

    help = "Mint ARKs in bulk"

    def add_arguments(self, parser):
        parser.add_argument("ark_count", type=int)
        parser.add_argument("naan", type=int)
        parser.add_argument("shoulder", type=str)

    def handle(self, *args, **options):
        ark_count = options["ark_count"]
        naan_id = options["naan"]
        naan = Naan.objects.get(pk=options["naan"])
        shoulder = options["shoulder"]

        Ark.objects.bulk_create(
            Ark(
                ark=f"{naan_id}{shoulder}{generate_noid(20)}",
                naan=naan,
                shoulder=shoulder,
            )
            for _ in range(ark_count)
        )
        self.stdout.write(self.style.SUCCESS(f"Successfully minted {ark_count} ARKs"))



"""Django Admin models for Arklet."""

from django.contrib import admin

from ark.models import Ark, Key, Naan, Shoulder, User


@admin.register(User)
class UserAdmin(admin.ModelAdmin):
    """Django Admin model for ARK admin users."""


@admin.register(Naan)
class NaanAdmin(admin.ModelAdmin):
    """Django Admin model for Name Assignment Authority Number bearing organizations."""

    list_display = ["name", "naan"]


@admin.register(Shoulder)
class ShoulderAdmin(admin.ModelAdmin):
    """Django Admin model for ARK shoulders."""

    list_display = ["shoulder", "name", "naan"]


@admin.register(Ark)
class ArkAdmin(admin.ModelAdmin):
    """Django Admin model for ARKs.

    In practice, stock Django Admin doesn't work well for large randomly sorted tables.
    This view will tend to take a long time to load as a full table count query is run.
    """

    list_display = ["ark", "url"]
    show_full_result_count = False


@admin.register(Key)
class KeyAdmin(admin.ModelAdmin):
    """Django Admin model for managing Arklet access keys.

    These access keys are used to mint and bind ARKs via the Arklet API.
    """

    list_display = ["key", "naan", "active"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ark',
 'ark.management',
 'ark.management.commands',
 'ark.migrations',
 'ark_import']

package_data = \
{'': ['*']}

extras_require = \
{':python_version >= "3.7" and python_version < "3.8"': ['Django>=3.2,<3.3'],
 ':python_version >= "3.8"': ['Django>=3.2'],
 'environ': ['django-environ>=0.8.1,<0.9.0'],
 'postgres': ['psycopg2>=2.7,<3.0'],
 'sentry': ['sentry-sdk>=1.5.12,<2.0.0']}

entry_points = \
{'console_scripts': ['createsuperuser = server:createsuperuser',
                     'migrate = server:migrate',
                     'server = server:main']}

setup_kwargs = {
    'name': 'arklet',
    'version': '0.1.0',
    'description': 'An unassuming ARK minter, binder, and resolver',
    'long_description': '# Arklet - A basic ARK resolver\n\n![lint_python](https://github.com/internetarchive/arklet/actions/workflows/lint_python.yml/badge.svg)\n\n\n## What is an ARK?\nSee https://arks.org/\n\n## What is Arklet?\nArklet is a Python Django application for minting, binding, and resolving ARKs.\nIt is intended to follow best practices set out by https://arks.org/.\n\nTechnical design notes:\n- Django is the only required dependency.\n- Supports each Django and Python version that is itself supported by the maintainers (Python 3.7-3.10, Django 3.2.x-4.0.x).\n    - Tests are run against the upcoming versions of Python and Django as well. \n- This repo can be run as a standalone service \n- ...or the ark package can be installed as a reusable app in other Django projects. \n    - If using the included arklet/settings.py file django-environ is also required.\n- Arklet is database agnostic.\n\nArklet is developed with poetry, pytest, black, tox, and more.\n\n## Running\n\n### Locally with Postgres\n\nCreate the default `.env` file in the project\'s root directory\n\n```\n# /!\\ Set your own secret key /!\\\nARKLET_DJANGO_SECRET_KEY=[YOUR_SECRET]\n\n# For local development, set to True\nARKLET_DEBUG=True\n```\n\nThe following steps walk through running Postgres, installing with poetry, and starting\nthe app. You can omit any of the extras listed in the poetry install step if they are\nnot used in your deployment. The included arklet/settings.py file does require environ.\nYou can skip installing the development dependencies by passing `--no-dev` to poetry.\nDjango is the only required dependency.\n```\ncd path/to/project\nmkdir postgres-data\ndocker run --name arklet-postgres -v postgres-data:/var/lib/postgresql/data \\\n    -p 5432:5432 \\\n    -e POSTGRES_USER=arklet -e POSTGRES_PASSWORD=arklet \\\n    -d postgres\npoetry install --extras "postgres sentry environ"\npoetry run python manage.py migrate\npoetry run python manage.py createsuperuser\npoetry run python manage.py runserver\n```\n\n### Separate dockers\nUsing docker, we can use a [this provided](./docker/env.docker.local) config file.\n\nSee above for running PostgreSQL, and run the **Arklet** docker as follows (in *bash*):\n```\ndocker build \\\n    --target dev \\\n    -t "arklet" -f ./Dockerfile . \\\n    --build-arg ENV=DEV \\\n&& docker run --rm -it \\\n    -p 8000:8000 \\\n    --env-file=./docker/env.docker.local \\\n    -e ARKLETDEBUG="true" \\\n    --name arklet \\\n    -v `pwd`/ark:/app/ark \\\n    -v `pwd`/ark_import:/app/ark_import \\\n    -v `pwd`/arklet:/app/arklet \\\n    arklet\n```\n\n### With docker-compose\nUsing the provided `docker-compose.yml` with default settings in the [docker\nconfiguration directory](./docker) :\n\n```\ndocker-compose up\n```\n\nBy default, the folders `ark`, `ark_import` and `arklet` are mounted in the\ncontainer. Should you wish to attach a console to the `arklet` container (needed\nto create the django superuser) :\n```\n# In another shell\ndocker exec -it arklet_django /bin/bash\n# You\'re now in the docker container\n./manage.py createsuperuser\n```\n\n### First steps\nCreate your first NAAN, Key, and Shoulder in the admin:\n127.0.0.1:8000/admin\n\nAnd by the way, you now host a working ARK resolver! You can already\ntry the following ones :\n- [http://127.0.0.1:8000/ark:/13960/t5n960f7n](http://127.0.0.1:8000/ark:/13960/t5n960f7n)\n- [http://127.0.0.1:8000/ark:/67375/C0X-SPWFRSGR-N](http://127.0.0.1:8000/ark:/67375/C0X-SPWFRSGR-N)\n- [http://127.0.0.1:8000/ark:/12148/bpt6k65358454](http://127.0.0.1:8000/ark:/12148/bpt6k65358454)\n- ...\n\nHappy minting, binding, and resolving!\n\n## Configuration Options\n\nSee arklet/settings.py for the full list of options to put in your config file.\n\n## Deploying\n### With docker\nUsing the provided Dockerfile (is you wish to set a build target, use `prod`, \nbut being the default target you can skip this), provide the following values\nin your environment:\n\n- ARKLET_DJANGO_SECRET_KEY=[YOUR_SECRET]\n- ARKLET_DEBUG=False\n- ARKLET_HOST=0.0.0.0\n- ARKLET_PORT=[Port of choice]\n- ARKLET_POSTGRES_NAME=[DB NAME]\n- ARKLET_POSTGRES_USER=[DB USER]\n- ARKLET_POSTGRES_PASSWORD=[DB PASS]\n- ARKLET_POSTGRES_HOST=[DB HOST]\n- ARKLET_POSTGRES_PORT=[DB PORT]\n',
    'author': 'Alex Dempsey',
    'author_email': 'avdempsey@archive.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/internetarchive/arklet',
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<4.0',
}


setup(**setup_kwargs)



"""Generate SQL queries from Noid/Egg Berkeley DB dump.

This script formats a Noid/Egg db dump into a series of SQL queries,
grouped into multiple files, suitable for importing into arklet.

As written, will only work for naan 13960 and shoulders /t, /fk.
Modify the extract_ark function to work for your DB file.

Supply a path to the db dump and a prefix for the query files.

Example call:
python -m ark_import sample_noid_output.txt output-prefix
"""

import sys

infile = sys.argv[1]
out_prefix = sys.argv[2]
queries_per_file = 10000


def signal_line(line):
    return line.startswith("ark:/") and line.endswith("_t")


def extract_ark(line):
    ark, record_type = line.split("|")
    proto, naan, number = ark.split("/")
    naan = int(naan)
    if naan != 13960:
        raise ValueError(f"bad naan: {naan}")
    if number.startswith("t"):
        shoulder = "/t"
        number = number[1:]
    elif number.startswith("fk"):
        shoulder = "/fk"
        number = number[2:]
    else:
        raise ValueError(number)
    if len(number) != 8:
        raise ValueError(f"unexpected number: {number}")
    return naan, shoulder, number


def query_format(naan, shoulder, number, url):
    ark = f"{naan}{shoulder}{number}"
    return repr((ark, shoulder, number, url, naan))


def ark_input_iter(ark_input_file):
    with open(ark_input_file, "r") as f:
        take_next = False
        for line in f:
            line = line.strip()
            if signal_line(line):
                take_next = True
                naan, shoulder, number = extract_ark(line)
            elif take_next:
                yield query_format(naan, shoulder, number, line)
                take_next = False
            else:
                take_next = False


def write_query_values(prefix, file_num, vals):
    # noinspection SqlDialectInspection,SqlNoDataSourceInspection
    head = "insert into ark_ark (ark, shoulder, assigned_name, url, naan_id) values "
    with open(f"{prefix}-{file_num:0>6}.sql", "w") as f:
        query_tail = ",\n".join(vals)
        query = f"{head}\n{query_tail};"
        f.write(query)


query_vals = []
outfile_num = 0
for query_val in ark_input_iter(infile):
    query_vals.append(query_val)
    if len(query_vals) >= queries_per_file:
        write_query_values(out_prefix, outfile_num, query_vals)
        query_vals.clear()
        outfile_num += 1
write_query_values(out_prefix, outfile_num, query_vals)  # write remaining values
